#!/usr/bin/env python

import os
from traceback import print_exc
import transmissionrpc

try:
    tc = transmissionrpc.Client(
        address=os.environ.get('TRANSMISSION_HOST', '127.0.0.1'),
        port=os.environ.get('TRANSMISSION_PORT', '9091'),
        user=os.environ.get('TRANSMISSION_USER', 'transmission'),
        password=os.environ.get('TRANSMISSION_PASSWORD', 'transmission'),
        timeout=2.0,
    )
except transmissionrpc.error.TransmissionError as e:
    message = e.message
    if e.original:
        message = '{e.message} Original reason: {e.original.message}.'.format(e=e)
    print('Failed to connect to transmission daemon: {message}'.format(message=message))
    exit(1)
except Exception as e:
    print('Unexpected error while trying to connect to transmission daemon. Raw error traceback:')
    print_exc()
    exit(2)

for torrent in tc.get_torrents():
    remove = False
    files = torrent.files()
    for id in files:
        download_dir = os.path.normpath(torrent.downloadDir)
        filename = files[id]['name']

        if not os.path.exists(os.path.join(download_dir, filename)) and torrent.status != 'downloading':
            remove = True
    if remove:
        tc.remove_torrent(torrent.id, delete_data=False)
