from json import dumps
import hashlib
import os

from django.conf import settings
from django.db import IntegrityError
from django.http import HttpResponse, HttpResponseBadRequest
from django.views.decorators.csrf import csrf_exempt

from core.models import Artefact


@csrf_exempt
def add(request):
    if request.method == 'POST':
        try:
            a = Artefact(
                name=request.POST.get('name'),
                magnet=request.POST.get('magnet'),
                size=request.POST.get('size')
            )

            a.save()
        except IntegrityError:
            return HttpResponse(content=dumps({
                'status': 'error',
                'info': 'already exists',
                'name': a.name,
                'magnet': a.magnet,
                'size': a.size
            }), status=409, content_type='application/json')

        return HttpResponse(content=dumps({
            'status': 'ok',
            'info': 'created',
            'name': a.name,
            'magnet': a.magnet,
            'size': a.size
        }), status=201, content_type='application/json')

    return HttpResponseBadRequest(content=dumps({
        'status': 'error',
        'info': 'POST method only is allowed'
    }), content_type='application/json')


def list(request):
    limit = request.GET.get('limit', 100)
    artefacts = Artefact.objects.all()[:limit]

    return HttpResponse(content=dumps([
        {'name': x.name, 'magnet': x.magnet} for x in artefacts
    ]), content_type='application/json')


@csrf_exempt
def publish(request, filename):
    if request.method == 'POST':
        if os.path.exists(os.path.join(settings.PUBLIC_STORAGE, filename)):
            return HttpResponse(content='Already exists', status=409)

        file = os.path.join(settings.PUBLIC_STORAGE, filename)
        torrent = os.path.join(
            settings.PUBLIC_STORAGE, '%s.torrent' % filename)

        fp = open(file, 'wb+')
        tp = open(torrent, 'wb+')

        try:
            for chunk in request.FILES['file'].chunks():
                fp.write(chunk)
        except KeyError:
            try:
                os.unlink(file)
            except:
                pass

            return HttpResponseBadRequest(content=dumps({
                'status': 'error',
                'info': "POST'ed file named file is required"
            }), content_type='application/json')

        try:
            for chunk in request.FILES['torrent'].chunks():
                tp.write(chunk)
        except KeyError:
            try:
                os.unlink(file)
            except:
                pass

            try:
                os.unlink(torrent)
            except:
                pass

            return HttpResponseBadRequest(content=dumps({
                'status': 'error',
                'info': "POST'ed file named torrent is required"
            }), content_type='application/json')

        fp.close()
        tp.close()

        try:
            checksum = request.POST.get('md5')

            h = hashlib.new('md5')
            h.update(open(file, 'rb').read())

            if h.hexdigest() != checksum:
                return HttpResponse(content=dumps({
                    'status': 'error',
                    'info': 'MD5 checksum is not matched'
                }), status=412, content_type='application/json')
        except KeyError:
            pass

        return HttpResponse(content=dumps({
            'status': 'ok',
            'info': 'created'
        }), status=201, content_type='application/json')
    else:
        return HttpResponseBadRequest(content=dumps({
            'status': 'error',
            'info': 'POST method only is allowed'
        }), content_type='application/json')
