#!/bin/bash

set -ex

setup_env_variables() {
    transmission-daemon
    # Wait for transmission daemon to start (10 seconds or less)
    for i in {1..10}; do
        nc -z 127.0.0.1 9091 && break
        sleep 1
    done

    export TRANSMISSION_HOST='127.0.0.1'
    export TRANSMISSION_PORT='9091'
    export TRANSMISSION_USER='transmission'
    export TRANSMISSION_PASSWORD='transmission'

    export TRACKER_URL='http://localhost:4994/announce'
    export HTTP_ROOT='http://localhost:4995/http_root/'
    export RANDOMFILE_NAME='myrandomfile.iso'

    export RANDOMFILE="/tmp/${RANDOMFILE_NAME}"
}

test_torrent() {
    echo "Running seedclient-wrapper torrent download test.."

    dd if=/dev/urandom of="${RANDOMFILE}" count=1 bs=1M

    LINK=`./client/seedclient.py -uvf "${RANDOMFILE}"`

    ./cleaner/seedcleaner.py

    # File is placed locally, should be downloaded successfully by its magnet link
    cd ./client
    if [[ $(timeout 5 ./seedclient-wrapper -dvm "${LINK}") != "${RANDOMFILE}" ]] ; then
      remove_rand_file
      cd -
      echo 'seedclient-wrapper output is not valid' >&2

      false
    fi

    cd -
    remove_rand_file
}

test_http() {
    echo "Running seedclient-wrapper HTTP download test.."

    dd if=/dev/urandom of="${RANDOMFILE}" count=1 bs=1M

    LINK=`./client/seedclient.py -uvf "${RANDOMFILE}"`

    ./cleaner/seedcleaner.py

    # Broke the link for torrent download fail and fallback to http download
    LINK=${LINK//btih/fake}

    # Simple local web server to serve a file from the /tmp directory
    cd /tmp/
    python -m SimpleHTTPServer &
    SERVER_PID=$!
    sleep 1
    cd -

    # Add fake web seed link on file placed locally
    LINK="${LINK}&ws=http%3A%2F%2F0.0.0.0%3A8000%2F${RANDOMFILE_NAME}"

    cd ./client
    if [[ $(timeout 5 ./seedclient-wrapper -dvm "${LINK}") != "${RANDOMFILE}" ]] ; then
      kill "${SERVER_PID}"

      remove_rand_file
      cd -
      echo 'seedclient-wrapper output is not valid' >&2

      false
    else
      kill "${SERVER_PID}"
    fi

    cd -
    remove_rand_file
}

seed_web_tests() {
    echo "seedclient tests is a go"

    dd if=/dev/urandom of="${RANDOMFILE}" count=1 bs=1M

    LINK=`./client/seedclient.py -uvf "${RANDOMFILE}"`

    remove_rand_file

    ./cleaner/seedcleaner.py

    timeout 5 ./client/seedclient.py -dvm "${LINK}" || (
    if [[ "$?" == "124" ]] ; then
      true
    else
      false
    fi )

    echo "seed-web tests is a go"

    ./seed/manage.py test
}

remove_rand_file() {
    rm -vf "${RANDOMFILE}"*""
}

clean_up() {
    remove_rand_file
    # stop transmission-daemon to prevent 'already running'
    # failures on additional patchset builds
    service transmission-daemon stop
}

functional_tests() {
    setup_env_variables

    test_torrent
    test_http

    seed_web_tests
    clean_up
}

syntax_tests() {
    flake8 --show-source src/
}

case "$1" in
    "syntax")
        syntax_tests
    ;;
    "functional")
        functional_tests
    ;;
    *)
        functional_tests
    ;;
esac
